<?php
/**
 * Plugin Name:       Cab fare calculator
 * Plugin URI:        https://kanev.com/products/taxi-booking-for-wordpress
 * Description:       Taxi Booking for Wordpress.
 * Version:           1.0.3
 * Author:            Marian Kanev
 * Author URI:        https://kanev.com/
 * License:           GPL v2 or later
 * License URI:       https://www.gnu.org/licenses/gpl-2.0.html
 */

if ( ! class_exists( 'WP_List_Table' ) ) {
    require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
}

define( 'TBLIGHT_PLUGIN_PATH', plugin_dir_path( __FILE__ ) );
define( 'TBLIGHT_PLUGIN_DIR_URL', plugin_dir_url( __FILE__ ) );

$tblight_db_version = 1; // it will be increased if any change in DB
define( 'TBLIGHT_DB_VERSION', $tblight_db_version );

register_activation_hook( __FILE__, 'tblight_install' );

function tblight_install() {

    global $wpdb;

    $installed_db_ver = (int)get_option( "tblight_db_version" );

    if($installed_db_ver == 0) // fresh install
    {
        require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
    
        $sql = file_get_contents(TBLIGHT_PLUGIN_PATH . 'admin/sql/install/cars.sql');
        $sql = str_replace('{WPDB_PREFIX}', $wpdb->prefix, $sql);
        dbDelta( $sql );
    
        $sql = file_get_contents(TBLIGHT_PLUGIN_PATH . 'admin/sql/install/configs.sql');
        $sql = str_replace('{WPDB_PREFIX}', $wpdb->prefix, $sql);
        dbDelta( $sql );
    
        $sql = file_get_contents(TBLIGHT_PLUGIN_PATH . 'admin/sql/install/countries.sql');
        $sql = str_replace('{WPDB_PREFIX}', $wpdb->prefix, $sql);
        dbDelta( $sql );
    
        $sql = file_get_contents(TBLIGHT_PLUGIN_PATH . 'admin/sql/install/currencies.sql');
        $sql = str_replace('{WPDB_PREFIX}', $wpdb->prefix, $sql);
        dbDelta( $sql );
    
        $sql = file_get_contents(TBLIGHT_PLUGIN_PATH . 'admin/sql/install/orders.sql');
        $sql = str_replace('{WPDB_PREFIX}', $wpdb->prefix, $sql);
        dbDelta( $sql );
    
        $sql = file_get_contents(TBLIGHT_PLUGIN_PATH . 'admin/sql/install/order_car.sql');
        $sql = str_replace('{WPDB_PREFIX}', $wpdb->prefix, $sql);
        dbDelta( $sql );
    
        $sql = file_get_contents(TBLIGHT_PLUGIN_PATH . 'admin/sql/install/paymentmethods.sql');
        $sql = str_replace('{WPDB_PREFIX}', $wpdb->prefix, $sql);
        dbDelta( $sql );
    
        $sql = file_get_contents(TBLIGHT_PLUGIN_PATH . 'admin/sql/install/payment_plg_cash.sql');
        $sql = str_replace('{WPDB_PREFIX}', $wpdb->prefix, $sql);
        dbDelta( $sql );
    
        $sql = file_get_contents(TBLIGHT_PLUGIN_PATH . 'admin/sql/install/payment_plg_paypal.sql');
        $sql = str_replace('{WPDB_PREFIX}', $wpdb->prefix, $sql);
        dbDelta( $sql );

        // now lets import data
        $sql = file_get_contents(TBLIGHT_PLUGIN_PATH . 'admin/sql/install/configs_data.sql');
        $sql = str_replace('{WPDB_PREFIX}', $wpdb->prefix, $sql);
        dbDelta( $sql );

        $sql = file_get_contents(TBLIGHT_PLUGIN_PATH . 'admin/sql/install/countries_data.sql');
        $sql = str_replace('{WPDB_PREFIX}', $wpdb->prefix, $sql);
        dbDelta( $sql );

        $sql = file_get_contents(TBLIGHT_PLUGIN_PATH . 'admin/sql/install/currencies_data.sql');
        $sql = str_replace('{WPDB_PREFIX}', $wpdb->prefix, $sql);
        dbDelta( $sql );

        add_option( 'tblight_db_version', TBLIGHT_DB_VERSION );
    }
    else // update
    {
        if($installed_db_ver != TBLIGHT_DB_VERSION)
        {
            $installed_db_ver++;
            
            for($folder_counter = $installed_db_ver; $folder_counter <= TBLIGHT_DB_VERSION; $folder_counter++)
            {
                $update_sql_dir = TBLIGHT_PLUGIN_PATH . "admin/sql/updates/$folder_counter";

                if (is_dir($update_sql_dir)) 
                {
                    if ($dh = opendir($update_sql_dir)) 
                    {
                        while (($file = readdir($dh)) !== false) 
                        {
                            if ($file != "." && $file != "..") 
                            {
                                $sql = file_get_contents(TBLIGHT_PRO_PLUGIN_PATH . "admin/sql/updates/$folder_counter/$file");
                                $sql = str_replace('{WPDB_PREFIX}', $wpdb->prefix, $sql);
                                dbDelta( $sql );
                            }
                        }
                        closedir($dh);
                    }
                }
            }

            update_option( 'tblight_db_version', TBLIGHT_DB_VERSION );
        }
    }
}

class TBLight_Plugin {

    const PLUGIN_INFO_JSON_URL = 'https://kanev.com/media/downloads/taxibooking-wordpress/wptblight-free-info.json';

	const PLUGIN_SLUG = 'wptblight-free';

    // class instance
    static $instance;

    // car WP_List_Table object
    public $cars_obj;

    // payment method WP_List_Table object
    public $paymentmethods_obj;    

    // payment method WP_List_Table object
    public $configs_obj;

    // order WP_List_Table object
    public $orders_obj;    

    // class constructor
    public function __construct() {
        add_filter( 'set-screen-option', [ __CLASS__, 'set_screen' ], 10, 3 );
        add_action( 'admin_menu', [ $this, 'tblight_plugin_menu' ] );
        add_action( 'admin_print_styles', [ $this, 'add_tblight_stylesheet' ] );
        add_action( 'admin_print_scripts', [ $this, 'add_tblight_scripts' ] );
    }

    public static function set_screen( $status, $option, $value ) {
        return $value;
    }

    public function add_tblight_stylesheet()
    {
        wp_enqueue_style( 'jquery-ui-datepicker-style', plugins_url( '/admin/css/jquery-ui.min.css', __FILE__ ) );
        wp_enqueue_style( 'tblight-style', plugins_url( '/admin/css/tblight_style.min.css', __FILE__ ) );
    }

    public function add_tblight_scripts()
    {
        wp_enqueue_script('jquery-ui-datepicker');
        wp_enqueue_media();
        wp_enqueue_script( 'custom-uploader-js', plugins_url( '/admin/js/uploader.js', __FILE__ ) );
        wp_enqueue_script('jquery-ui-tooltip');
        wp_enqueue_script('jquery-ui-tabs');
        wp_enqueue_script( 'lord-icons', plugins_url( '/admin/js/lord-icon.js', __FILE__ ) );

        require_once(TBLIGHT_PLUGIN_PATH . 'classes/booking.helper.php');
        $elsettings = BookingHelper::config();
        $gmap_api_key = $elsettings->api_key;   
        if(!empty($_GET['page']) && $_GET['page'] == "configs" && !empty($_GET['action']) && $_GET['action'] == "edit" && !empty($_GET['id']) && $_GET['id'] == 4 ) {         
            if ($gmap_api_key!="") {
                wp_enqueue_script( 'maps-googleapis', '//maps.googleapis.com/maps/api/js?v=3&key='.$gmap_api_key );
            } else {
                wp_enqueue_script( 'maps-googleapis', '//maps.googleapis.com/maps/api/js?v=3' );
            }
            wp_enqueue_script( 'google-maps-getbounds-js', plugins_url( 'assets/js/google.maps.Polygon.getBounds.js', __FILE__ ) );
            wp_enqueue_script( 'google-maps-moveto-js', plugins_url( 'assets/js/google.maps.Polygon.moveTo.js', __FILE__ ) );  
        } elseif (!empty($_GET['page']) && $_GET['page'] == "orders" && !empty($_GET['action']) && $_GET['action'] == "edit") {
            $gmap_api_key = $elsettings->api_key;            
            if ($gmap_api_key!="") {
                wp_enqueue_script( 'maps-googleapis', '//maps.googleapis.com/maps/api/js?v=3&language=en&libraries=geometry,places&key='.$gmap_api_key );
            } else {
                wp_enqueue_script( 'maps-googleapis', '//maps.googleapis.com/maps/api/js?v=3' );
            }
        }      
    }    

    public function tblight_plugin_menu() {
        
        add_menu_page(
            'Taxi Booking',
            'Taxi Booking',
            'manage_options',
            'dashboard',
            [ $this, 'tblight_dashboard_page' ],
            'dashicons-businessperson',
            20
        );

        // Orders menu item
        require_once(TBLIGHT_PLUGIN_PATH . 'admin/classes/orders_list.php');
        
        $orders_hook = add_submenu_page(
            'dashboard',
            'TaxiBooking - Orders',
            'Orders',
            'manage_options',
            'orders',
            [ $this, 'tblight_orders_page' ]
        );
        add_action( "load-$orders_hook", [ $this, 'screen_option' ] );
        // Orders menu item          
        
        // Cars menu item
        require_once(TBLIGHT_PLUGIN_PATH . 'admin/classes/cars_list.php');
        
        $hook = add_submenu_page(
            'dashboard',
            'TaxiBooking - Cars',
            'Vehicles',
            'manage_options',
            'cars',
            [ $this, 'tblight_cars_page' ]
        );
        add_action( "load-$hook", [ $this, 'screen_option' ] );
        // Cars menu item
        
        // PaymentMethods menu item
        require_once(TBLIGHT_PLUGIN_PATH . 'admin/classes/paymentmethods_list.php');
        
        $paymentmethods_hook = add_submenu_page(
            'dashboard',
            'TaxiBooking - Payment Methods',
            'Payment Methods',
            'manage_options',
            'paymentmethods',
            [ $this, 'tblight_paymentmethods_page' ]
        );
        add_action( "load-$paymentmethods_hook", [ $this, 'screen_option' ] );
        // PaymentMethods menu item

        // Configs menu item
        require_once(TBLIGHT_PLUGIN_PATH . 'admin/classes/configs_list.php');
        
        $configs_hook = add_submenu_page(
            'dashboard',
            'TaxiBooking - Settings',
            'Settings',
            'manage_options',
            'configs',
            [ $this, 'tblight_configs_page' ]
        );
        add_action( "load-$configs_hook", [ $this, 'screen_option' ] );
        // Configs menu item               
    }
    
    public function tblight_dashboard_page()
    {
        if ( is_admin() ) {
            if( ! function_exists( 'get_plugin_data' ) ) {
                require_once( ABSPATH . 'wp-admin/includes/plugin.php' );
            }
            $plugin_data = get_plugin_data( __FILE__ );
        }

        require_once(TBLIGHT_PLUGIN_PATH . 'admin/views/dashboard.php');
    }
    public function tblight_cars_page()
    {
        require_once(TBLIGHT_PLUGIN_PATH . 'admin/controllers/car.php');
    }
    public function tblight_paymentmethods_page()
    {
        require_once(TBLIGHT_PLUGIN_PATH . 'admin/controllers/paymentmethod.php');
    }
    public function tblight_configs_page()
    {
        require_once(TBLIGHT_PLUGIN_PATH . 'admin/controllers/config.php');
    }
    public function tblight_orders_page()
    {
        require_once(TBLIGHT_PLUGIN_PATH . 'admin/controllers/order.php');
    }

    /**
     * Screen options
     */
    public function screen_option() {
    
        $option = 'per_page';
        $args   = [
            'label'   => 'Number of items per page:',
            'default' => 20,
            'option'  => 'items_per_page'
        ];
        
        if ($_REQUEST['page']=='cars') {
            add_screen_option( $option, $args );
            $this->cars_obj = new Cars_List();
    	}
        if ($_REQUEST['page']=='paymentmethods') {
            add_screen_option( $option, $args );
            $this->paymentmethods_obj = new Paymentmethods_List();
        }  
        if ($_REQUEST['page']=='configs') {
            $this->configs_obj = new Configs_List();
        } 
        if ($_REQUEST['page']=='orders') {
            add_screen_option( $option, $args );
            $this->orders_obj = new Orders_List();
        }        
    }

    /** Singleton instance */
    public static function get_instance() {
        if ( ! isset( self::$instance ) ) {
            self::$instance = new self();
        }

        return self::$instance;
    }
}

add_action( 'plugins_loaded', function () {
    TBLight_Plugin::get_instance();

    add_action('wp_enqueue_scripts', 'tblight_include_custom_jquery');

    if(!empty($_GET['pg']) && $_GET['pg'] == "thanks"){
        add_shortcode('taxibooking-form', 'show_booking_thanks');
    }
    else {
        add_shortcode('taxibooking-form', 'show_booking_form');
    }
});

function app_output_buffer() {
    ob_start();
} // soi_output_buffer
add_action('init', 'app_output_buffer');

// include custom jQuery
function tblight_include_custom_jquery() {

    wp_deregister_script('jquery');
    wp_enqueue_script('jquery', 'https://ajax.googleapis.com/ajax/libs/jquery/1.9.1/jquery.min.js', array(), null, true);
}

// Frontend codes
function show_booking_form() {
    
    require_once(TBLIGHT_PLUGIN_PATH . 'classes/booking.helper.php');
    require_once(TBLIGHT_PLUGIN_PATH . 'fields/select.list.php');
    
    $elsettings = BookingHelper::config();
    $offset  = get_option( 'gmt_offset' );
    //$booking_form_url = $_SERVER['REQUEST_URI'];

    global $wp;
	$booking_form_url = home_url( add_query_arg( array(), $wp->request ) );

    $countryObj = CompanyHelper::getCountryById($elsettings->default_country);
    $default_country = !empty($countryObj) ? $countryObj->country_2_code : '';
    
    wp_enqueue_style( 'tblight-custom-bootstrap-style', plugins_url( '/assets/css/custom-bootstrap.min.css', __FILE__ ) );
    wp_enqueue_style( 'tblight-main-style', plugins_url( '/assets/css/main.min.css', __FILE__ ) );
    wp_enqueue_style( 'tblight-media-style', plugins_url( '/assets/css/media.min.css', __FILE__ ) );
    wp_enqueue_style( 'jquery-chosen-css', plugins_url( '/assets/css/chosen.min.css', __FILE__ ) );
    wp_enqueue_style( 'tblight-fontawesome-style', plugins_url( '/assets/css/font-awesome.min.css', __FILE__ ) );
    wp_enqueue_style( 'tblight-font-family', 'https://fonts.googleapis.com/css?family=Muli' );
 
    require_once(TBLIGHT_PLUGIN_PATH . 'views/tblight/default.php');

    //wp_enqueue_script( 'jquery' ); //v=1.9.1
    wp_enqueue_script( 'jquery-chosen-js', plugins_url( '/assets/js/chosen.jquery.min.js', __FILE__ ) ); //v=1.8.7
    wp_enqueue_script( 'jquery-moment-js', plugins_url( '/assets/js/moment-with-locales.min.js', __FILE__ ) ); //v=2.24.0
    wp_enqueue_script( 'tblight-custom-bootstrap-js', plugins_url( '/assets/js/custom-bootstrap.min.js', __FILE__ ) );
    wp_enqueue_script( 'bootstrap-datetimepicker-js', plugins_url( '/assets/js/bootstrap-datetimepicker.min.js', __FILE__ ) );
    wp_enqueue_script( 'tblight-core-js', plugins_url( '/assets/js/tbfcore.min.js', __FILE__ ) );
    wp_enqueue_script( 'tblight-googlegeo-js', plugins_url( '/assets/js/googleGeo.min.js', __FILE__ ) );
    
    wp_enqueue_script('google-maps', 'https://maps.googleapis.com/maps/api/js?v=3&language=en&libraries=geometry,places&key='.trim($elsettings->api_key));
    
    wp_enqueue_script( 'tblight-main-js', plugins_url( '/assets/js/main.min.js', __FILE__ ) );
} 

function show_booking_thanks() {
    global $wp;

    require_once(TBLIGHT_PLUGIN_PATH . 'classes/booking.helper.php');
    $booking_form_url = home_url( add_query_arg( array(), $wp->request ) );
    require_once(TBLIGHT_PLUGIN_PATH . 'views/tblight/thanks.php');
} 

load_theme_textdomain( 'tblight', TBLIGHT_PLUGIN_PATH . '/languages' );

if(!empty($_GET['controller']) && !empty($_GET['action']) && !empty($_GET['ajax']) && $_GET['ajax'] == 1)
{
    require_once(TBLIGHT_PLUGIN_PATH . 'controllers/'.$_GET['controller'].'.php');
}
elseif(!empty($_GET['pg']) && $_GET['pg'] == "tblight" && !empty($_GET['controller']) && !empty($_GET['action'])) // This will be used for IPN
{
    require_once(TBLIGHT_PLUGIN_PATH . 'controllers/'.$_GET['controller'].'.php');
}